@@ -1,12 +1,14 @@
 extern crate bson;
-extern crate envy;
 extern crate flexbuffers;
+#[cfg(feature = "http")]
+extern crate http;
 extern crate json5;
 extern crate postcard;
 extern crate rmp_serde;
 extern crate ron;
 extern crate serde;
 extern crate serde_cbor;
+#[macro_use]
 extern crate serde_derive;
 extern crate serde_json;
 extern crate serde_lexpr;
@@ -14,16 +16,35 @@ extern crate serde_pickle;
 extern crate serde_qs;
 extern crate serde_xml_rs;
 extern crate serde_yaml;
-extern crate zvariant;
 
-use std::str::{from_utf8, Utf8Error};
+pub mod prelude {
+    pub extern crate bson;
+    pub extern crate flexbuffers;
+    pub extern crate json5;
+    pub extern crate postcard;
+    pub extern crate rmp_serde as messagepack;
+    pub extern crate ron;
+    pub extern crate serde_cbor as cbor;
+    pub extern crate serde_derive;
+    pub extern crate serde_json as json;
+    pub extern crate serde_lexpr as lexpr;
+    pub extern crate serde_pickle as pickle;
+    pub extern crate serde_qs as url;
+    pub extern crate serde_xml_rs as xml;
+    pub extern crate serde_yaml as yaml;
+}
 
-use byteorder::LE;
-use serde::__private::de::TagOrContentField::Content;
+#[cfg(feature = "http")]
+use crate::Error::InvalidHeaderValue;
+#[cfg(feature = "http")]
+use actix_http::header::TryIntoHeaderValue;
+use derive_more::Display;
+#[cfg(feature = "http")]
+use http::{header::ToStrError, HeaderValue};
 use serde::de::DeserializeOwned;
 use serde::Serialize;
 use std::convert::{Infallible, TryFrom, TryInto};
-use zvariant::{from_slice, to_bytes, EncodingContext as Context, Type};
+use std::str::Utf8Error;
 
 pub type Result<T> = std::result::Result<T, Error>;
 
@@ -57,6 +78,8 @@ impl TryFrom<&str> for ContentType {
             "application/cbor" => Ok(ContentType::Cbor),
             "application/x-cbor" => Ok(ContentType::Cbor),
             "flexbuffers" => Ok(ContentType::FlexBuffers),
+            "application/flexbuffers" => Ok(ContentType::FlexBuffers),
+            "application/x-flexbuffers" => Ok(ContentType::FlexBuffers),
             "json" => Ok(ContentType::Json),
             "application/json" => Ok(ContentType::Json),
             "application/x-json" => Ok(ContentType::Json),
@@ -67,8 +90,14 @@ impl TryFrom<&str> for ContentType {
             "application/lexpr" => Ok(ContentType::Lexpr),
             "application/x-lexpr" => Ok(ContentType::Lexpr),
             "messagepack" => Ok(ContentType::MessagePack),
+            "application/messagepack" => Ok(ContentType::MessagePack),
+            "application/x-messagepack" => Ok(ContentType::MessagePack),
             "pickle" => Ok(ContentType::Pickle),
+            "application/pickle" => Ok(ContentType::Pickle),
+            "application/x-pickle" => Ok(ContentType::Pickle),
             "postcard" => Ok(ContentType::Postcard),
+            "application/postcard" => Ok(ContentType::Postcard),
+            "application/x-postcard" => Ok(ContentType::Postcard),
             "ron" => Ok(ContentType::Ron),
             "application/ron" => Ok(ContentType::Ron),
             "application/x-ron" => Ok(ContentType::Ron),
@@ -76,6 +105,8 @@ impl TryFrom<&str> for ContentType {
             "application/toml" => Ok(ContentType::Toml),
             "application/x-toml" => Ok(ContentType::Toml),
             "url" => Ok(ContentType::Url),
+            "application/url" => Ok(ContentType::Url),
+            "application/x-url" => Ok(ContentType::Url),
             "yaml" => Ok(ContentType::Yaml),
             "application/yaml" => Ok(ContentType::Yaml),
             "application/x-yaml" => Ok(ContentType::Yaml),
@@ -87,31 +118,166 @@ impl TryFrom<&str> for ContentType {
     }
 }
 
-#[derive(Debug)]
-pub enum Error {
-    Infallible,
-    ByteToUTF8ConversionFailure(Utf8Error),
-    UnknownContentTypeMatchFromStr(String),
-    BsonSerializationFailure(bson::ser::Error),
-    BsonDeserializationFailure(bson::de::Error),
-    CborFailure(serde_cbor::Error),
-    FlexBuffersSerializationFailure(flexbuffers::SerializationError),
-    FlexBuffersDeserializationFailure(flexbuffers::DeserializationError),
-    JsonError(serde_json::Error),
-    Json5Error(json5::Error),
-    LexprError(serde_lexpr::Error),
-    MessagePackEncodeError(rmp_serde::encode::Error),
-    MessagePackDecodeError(rmp_serde::decode::Error),
-    PickleError(serde_pickle::Error),
-    PostcardError(postcard::Error),
-    RonError(ron::Error),
-    TomlSerializationFailure(toml::ser::Error),
-    TomlDeserializationFailure(toml::de::Error),
-    UrlEncodingFailure(serde_qs::Error),
-    YamlError(serde_yaml::Error),
-    XmlError(serde_xml_rs::Error),
-    TypeDoesNotSupportSerialization(ContentType),
+impl TryFrom<&ContentType> for ContentType {
+    type Error = crate::Error;
+
+    fn try_from(h: &ContentType) -> std::result::Result<ContentType, Self::Error> {
+        match h {
+            Self::Bson => Ok(Self::Bson),
+            Self::Cbor => Ok(Self::Cbor),
+            Self::FlexBuffers => Ok(Self::FlexBuffers),
+            Self::Json => Ok(Self::Json),
+            Self::Json5 => Ok(Self::Json5),
+            Self::Lexpr => Ok(Self::Lexpr),
+            Self::MessagePack => Ok(Self::MessagePack),
+            Self::Pickle => Ok(Self::Pickle),
+            Self::Postcard => Ok(Self::Postcard),
+            Self::Ron => Ok(Self::Ron),
+            Self::Toml => Ok(Self::Toml),
+            Self::Url => Ok(Self::Url),
+            Self::Yaml => Ok(Self::Yaml),
+            Self::Xml => Ok(Self::Xml),
+        }
+    }
 }
+
+#[cfg(feature = "http")]
+impl TryIntoHeaderValue for ContentType {
+    type Error = http::header::InvalidHeaderValue;
+
+    fn try_into_value(self) -> std::result::Result<HeaderValue, Self::Error> {
+        HeaderValue::from_str(match self {
+            ContentType::Bson => "application/x-bson",
+            ContentType::Cbor => "application/x-cbor",
+            ContentType::FlexBuffers => "application/x-flexbuffers",
+            ContentType::Json => "application/json",
+            ContentType::Json5 => "application/json5",
+            ContentType::Lexpr => "application/x-lexpr",
+            ContentType::MessagePack => "application/x-messagepack",
+            ContentType::Pickle => "application/x-pickle",
+            ContentType::Postcard => "application/x-postcard",
+            ContentType::Ron => "application/ron",
+            ContentType::Toml => "application/toml",
+            ContentType::Url => "application/x-url",
+            ContentType::Yaml => "application/yaml",
+            ContentType::Xml => "application/xml",
+        })
+    }
+}
+#[cfg(feature = "http")]
+impl TryIntoHeaderValue for &ContentType {
+    type Error = http::header::InvalidHeaderValue;
+
+    fn try_into_value(self) -> std::result::Result<HeaderValue, Self::Error> {
+        HeaderValue::from_str(match self {
+            ContentType::Bson => "application/x-bson",
+            ContentType::Cbor => "application/x-cbor",
+            ContentType::FlexBuffers => "application/x-flexbuffers",
+            ContentType::Json => "application/json",
+            ContentType::Json5 => "application/json5",
+            ContentType::Lexpr => "application/x-lexpr",
+            ContentType::MessagePack => "application/x-messagepack",
+            ContentType::Pickle => "application/x-pickle",
+            ContentType::Postcard => "application/x-postcard",
+            ContentType::Ron => "application/ron",
+            ContentType::Toml => "application/toml",
+            ContentType::Url => "application/x-url",
+            ContentType::Yaml => "application/yaml",
+            ContentType::Xml => "application/xml",
+        })
+    }
+}
+
+#[cfg(feature = "http")]
+impl TryFrom<HeaderValue> for ContentType {
+    type Error = Error;
+
+    fn try_from(h: HeaderValue) -> std::result::Result<ContentType, Self::Error> {
+        h.to_str()
+            .map_err(Error::from)
+            .and_then(ContentType::try_from)
+    }
+}
+
+#[cfg(feature = "http")]
+impl TryFrom<&HeaderValue> for ContentType {
+    type Error = Error;
+
+    fn try_from(h: &HeaderValue) -> std::result::Result<ContentType, Self::Error> {
+        h.to_str()
+            .map_err(Error::from)
+            .and_then(ContentType::try_from)
+    }
+}
+
+#[derive(Debug, Display)]
+pub enum Error {
+    #[display(fmt = "Infallible - This error should have been infallible")]
+    Infallible,
+    #[display(fmt = "Converting Raw Data to UTF8 failed: {}", _0)]
+    ByteToUTF8ConversionFailure(Utf8Error),
+    #[display(fmt = "Unknown content type match from str: {}", _0)]
+    UnknownContentTypeMatchFromStr(String),
+    #[display(fmt = "BSON encoder/decoder error: {}", _0)]
+    BsonSerializationFailure(bson::ser::Error),
+    #[display(fmt = "BSON encode/decoder error: {}", _0)]
+    BsonDeserializationFailure(bson::de::Error),
+    #[display(fmt = "CBOR encoder/decoder error: {}", _0)]
+    CborFailure(serde_cbor::Error),
+    #[display(fmt = "Flexbuffers encoder/decoder error: {}", _0)]
+    FlexBuffersSerializationFailure(flexbuffers::SerializationError),
+    #[display(fmt = "Flexbuffers encoder/decoder error: {}", _0)]
+    FlexBuffersDeserializationFailure(flexbuffers::DeserializationError),
+    #[display(fmt = "JSON encoder/decoder error: {}", _0)]
+    JsonError(serde_json::Error),
+    #[display(fmt = "JSON5 encoder/decoder error: {}", _0)]
+    Json5Error(json5::Error),
+    #[display(fmt = "LEXPR encoder/decoder error: {}", _0)]
+    LexprError(serde_lexpr::Error),
+    #[display(fmt = "MessagePack encoder/decoder error: {}", _0)]
+    MessagePackEncodeError(rmp_serde::encode::Error),
+    #[display(fmt = "MessagePack encoder/decoder error: {}", _0)]
+    MessagePackDecodeError(rmp_serde::decode::Error),
+    #[display(fmt = "Pickle encoder/decoder error: {}", _0)]
+    PickleError(serde_pickle::Error),
+    #[display(fmt = "Postcard encoder/decoder error: {}", _0)]
+    PostcardError(postcard::Error),
+    #[display(fmt = "RON encoder/decoder error: {}", _0)]
+    RonError(ron::Error),
+    #[display(fmt = "TOML encoder/decoder error: {}", _0)]
+    TomlSerializationFailure(toml::ser::Error),
+    #[display(fmt = "TOML encoder/decoder error: {}", _0)]
+    TomlDeserializationFailure(toml::de::Error),
+    #[display(fmt = "URL encoder/decoder error: {}", _0)]
+    UrlEncodingFailure(serde_qs::Error),
+    #[display(fmt = "YAML encoder/decoder error: {}", _0)]
+    YamlError(serde_yaml::Error),
+    #[display(fmt = "XML encoder/decoder error: {}", _0)]
+    XmlError(serde_xml_rs::Error),
+    #[display(fmt = "Type is not supported for encoding/decoding: {:?}", _0)]
+    TypeDoesNotSupportSerialization(ContentType),
+    #[cfg(feature = "http")]
+    #[display(fmt = "Failed to convert `HeaderValue` to a ContentType: {}", _0)]
+    FailedConvertingHeaderValueToContentType(http::header::ToStrError),
+    #[cfg(feature = "http")]
+    #[display(fmt = "Invalid Header Value found: {}", _0)]
+    InvalidHeaderValue(http::header::InvalidHeaderValue),
+}
+
+#[cfg(feature = "http")]
+impl From<http::header::InvalidHeaderValue> for Error {
+    fn from(e: http::header::InvalidHeaderValue) -> Self {
+        Self::InvalidHeaderValue(e)
+    }
+}
+
+#[cfg(feature = "http")]
+impl From<http::header::ToStrError> for Error {
+    fn from(e: ToStrError) -> Self {
+        Self::FailedConvertingHeaderValueToContentType(e)
+    }
+}
+
 impl From<std::convert::Infallible> for Error {
     fn from(_: Infallible) -> Self {
         Error::Infallible
